@@ -20,14 +20,14 @@ impl BuildValidator for ChainValidator {
     const EXPECTED_TYPE: &'static str = "chain";
 
     fn build(
-        schema: &PyDict,
-        config: Option<&PyDict>,
+        schema: &Bound<'_, PyDict>,
+        config: Option<&Bound<'_, PyDict>>,
         definitions: &mut DefinitionsBuilder<CombinedValidator>,
     ) -> PyResult<CombinedValidator> {
         let steps: Vec<CombinedValidator> = schema
-            .get_as_req::<&PyList>(intern!(schema.py(), "steps"))?
+            .get_as_req::<Bound<'_, PyList>>(intern!(schema.py(), "steps"))?
             .iter()
-            .map(|step| build_validator_steps(step, config, definitions))
+            .map(|step| build_validator_steps(&step, config, definitions))
             .collect::<PyResult<Vec<Vec<CombinedValidator>>>>()?
             .into_iter()
             .flatten()
@@ -54,9 +54,9 @@ impl BuildValidator for ChainValidator {
 
 // either a vec of the steps from a nested `ChainValidator`, or a length-1 vec containing the validator
 // to be flattened into `steps` above
-fn build_validator_steps<'a>(
-    step: &'a PyAny,
-    config: Option<&'a PyDict>,
+fn build_validator_steps(
+    step: &Bound<'_, PyAny>,
+    config: Option<&Bound<'_, PyDict>>,
     definitions: &mut DefinitionsBuilder<CombinedValidator>,
 ) -> PyResult<Vec<CombinedValidator>> {
     let validator = build_validator(step, config, definitions)?;
@@ -70,17 +70,17 @@ fn build_validator_steps<'a>(
 impl_py_gc_traverse!(ChainValidator { steps });
 
 impl Validator for ChainValidator {
-    fn validate<'data>(
+    fn validate<'py>(
         &self,
-        py: Python<'data>,
-        input: &'data impl Input<'data>,
-        state: &mut ValidationState,
+        py: Python<'py>,
+        input: &(impl Input<'py> + ?Sized),
+        state: &mut ValidationState<'_, 'py>,
     ) -> ValResult<PyObject> {
         let mut steps_iter = self.steps.iter();
         let first_step = steps_iter.next().unwrap();
         let value = first_step.validate(py, input, state)?;
 
-        steps_iter.try_fold(value, |v, step| step.validate(py, v.into_ref(py), state))
+        steps_iter.try_fold(value, |v, step| step.validate(py, v.bind(py), state))
     }
 
     fn get_name(&self) -> &str {
