@@ -68,15 +68,16 @@ def test_function_args():
     # insert_assert(f_info)
     assert f_info == {
         'mode': 'python',
-        'by_alias': True,
+        'by_alias': None,
         'exclude_unset': False,
         'exclude_defaults': False,
         'exclude_none': False,
         'round_trip': False,
+        'serialize_as_any': False,
     }
     assert s.to_python('x') == 'xx'
 
-    assert s.to_python(4, mode='foobar') == 8
+    assert s.to_python(4, mode='foobar', by_alias=True) == 8
     # insert_assert(f_info)
     assert f_info == {
         'mode': 'foobar',
@@ -85,17 +86,19 @@ def test_function_args():
         'exclude_defaults': False,
         'exclude_none': False,
         'round_trip': False,
+        'serialize_as_any': False,
     }
 
     assert s.to_json(42) == b'84'
     # insert_assert(f_info)
     assert f_info == {
         'mode': 'json',
-        'by_alias': True,
+        'by_alias': None,
         'exclude_unset': False,
         'exclude_defaults': False,
         'exclude_none': False,
         'round_trip': False,
+        'serialize_as_any': False,
     }
 
     assert s.to_python(7, mode='json', by_alias=False, exclude_unset=True) == 14
@@ -107,6 +110,7 @@ def test_function_args():
         'exclude_defaults': False,
         'exclude_none': False,
         'round_trip': False,
+        'serialize_as_any': False,
     }
 
     assert s.to_python(1, include={1, 2, 3}, exclude={'foo': {'bar'}}) == 2
@@ -115,11 +119,25 @@ def test_function_args():
         'include': {3, 2, 1},
         'exclude': {'foo': {'bar'}},
         'mode': 'python',
-        'by_alias': True,
+        'by_alias': None,
         'exclude_unset': False,
         'exclude_defaults': False,
         'exclude_none': False,
         'round_trip': False,
+        'serialize_as_any': False,
+    }
+
+    assert s.to_python(1, context='context') == 2
+    # insert_assert(f_info)
+    assert f_info == {
+        'context': 'context',
+        'mode': 'python',
+        'by_alias': None,
+        'exclude_unset': False,
+        'exclude_defaults': False,
+        'exclude_none': False,
+        'round_trip': False,
+        'serialize_as_any': False,
     }
 
 
@@ -189,7 +207,7 @@ def test_function_known_type():
     assert s.to_python([1, 2, 3], mode='json') == [1, 2, 3, 42]
     assert s.to_json([1, 2, 3]) == b'[1,2,3,42]'
 
-    msg = r'Expected `list\[int\]` but got `str` - serialized value may not be as expected'
+    msg = r"Expected `list\[int\]` - serialized value may not be as expected \[input_value='abc', input_type=str\]"
     with pytest.warns(UserWarning, match=msg):
         assert s.to_python('abc') == 'abc'
 
@@ -212,24 +230,28 @@ def test_function_args_str():
         )
     )
     assert s.to_python(123) == (
-        "123 info=SerializationInfo(include=None, exclude=None, mode='python', by_alias=True, exclude_unset=False, "
-        'exclude_defaults=False, exclude_none=False, round_trip=False)'
+        "123 info=SerializationInfo(include=None, exclude=None, context=None, mode='python', by_alias=False, exclude_unset=False, "
+        'exclude_defaults=False, exclude_none=False, round_trip=False, serialize_as_any=False)'
     )
     assert s.to_python(123, mode='other') == (
-        "123 info=SerializationInfo(include=None, exclude=None, mode='other', by_alias=True, exclude_unset=False, "
-        'exclude_defaults=False, exclude_none=False, round_trip=False)'
+        "123 info=SerializationInfo(include=None, exclude=None, context=None, mode='other', by_alias=False, exclude_unset=False, "
+        'exclude_defaults=False, exclude_none=False, round_trip=False, serialize_as_any=False)'
     )
     assert s.to_python(123, include={'x'}) == (
-        "123 info=SerializationInfo(include={'x'}, exclude=None, mode='python', by_alias=True, exclude_unset=False, "
-        'exclude_defaults=False, exclude_none=False, round_trip=False)'
+        "123 info=SerializationInfo(include={'x'}, exclude=None, context=None, mode='python', by_alias=False, exclude_unset=False, "
+        'exclude_defaults=False, exclude_none=False, round_trip=False, serialize_as_any=False)'
+    )
+    assert s.to_python(123, context='context') == (
+        "123 info=SerializationInfo(include=None, exclude=None, context='context', mode='python', by_alias=False, exclude_unset=False, "
+        'exclude_defaults=False, exclude_none=False, round_trip=False, serialize_as_any=False)'
     )
     assert s.to_python(123, mode='json', exclude={1: {2}}) == (
-        "123 info=SerializationInfo(include=None, exclude={1: {2}}, mode='json', by_alias=True, exclude_unset=False, "
-        'exclude_defaults=False, exclude_none=False, round_trip=False)'
+        "123 info=SerializationInfo(include=None, exclude={1: {2}}, context=None, mode='json', by_alias=False, exclude_unset=False, "
+        'exclude_defaults=False, exclude_none=False, round_trip=False, serialize_as_any=False)'
     )
     assert s.to_json(123) == (
-        b'"123 info=SerializationInfo(include=None, exclude=None, mode=\'json\', by_alias=True, exclude_unset=False, '
-        b'exclude_defaults=False, exclude_none=False, round_trip=False)"'
+        b"\"123 info=SerializationInfo(include=None, exclude=None, context=None, mode='json', by_alias=False, exclude_unset=False, "
+        b'exclude_defaults=False, exclude_none=False, round_trip=False, serialize_as_any=False)"'
     )
 
 
@@ -300,11 +322,20 @@ def test_wrong_return_type():
             )
         )
     )
-    with pytest.warns(UserWarning, match='Expected `int` but got `str` - serialized value may not be as expected'):
+    with pytest.warns(
+        UserWarning,
+        match=r"Expected `int` - serialized value may not be as expected \[input_value='123', input_type=str\]",
+    ):
         assert s.to_python(123) == '123'
-    with pytest.warns(UserWarning, match='Expected `int` but got `str` - serialized value may not be as expected'):
+    with pytest.warns(
+        UserWarning,
+        match=r"Expected `int` - serialized value may not be as expected \[input_value='123', input_type=str\]",
+    ):
         assert s.to_python(123, mode='json') == '123'
-    with pytest.warns(UserWarning, match='Expected `int` but got `str` - serialized value may not be as expected'):
+    with pytest.warns(
+        UserWarning,
+        match=r"Expected `int` - serialized value may not be as expected \[input_value='123', input_type=str\]",
+    ):
         assert s.to_json(123) == b'"123"'
 
 
@@ -334,11 +365,20 @@ def test_function_wrap_return_scheam():
     assert s.to_python(3) == 'result=3'
     assert s.to_python(3, mode='json') == 'result=3'
     assert s.to_json(3) == b'"result=3"'
-    with pytest.warns(UserWarning, match='Expected `str` but got `int` - serialized value may not be as expected'):
+    with pytest.warns(
+        UserWarning,
+        match=r'Expected `str` - serialized value may not be as expected \[input_value=42, input_type=int\]',
+    ):
         assert s.to_python(42) == 42
-    with pytest.warns(UserWarning, match='Expected `str` but got `int` - serialized value may not be as expected'):
+    with pytest.warns(
+        UserWarning,
+        match=r'Expected `str` - serialized value may not be as expected \[input_value=42, input_type=int\]',
+    ):
         assert s.to_python(42, mode='json') == 42
-    with pytest.warns(UserWarning, match='Expected `str` but got `int` - serialized value may not be as expected'):
+    with pytest.warns(
+        UserWarning,
+        match=r'Expected `str` - serialized value may not be as expected \[input_value=42, input_type=int\]',
+    ):
         assert s.to_json(42) == b'42'
 
 
@@ -588,8 +628,11 @@ def test_function_after_preserves_wrapped_serialization():
     def f(value, _info):
         return value
 
-    s = SchemaSerializer(core_schema.general_after_validator_function(f, core_schema.int_schema()))
-    with pytest.warns(UserWarning, match='Expected `int` but got `str` - serialized value may not be as expected'):
+    s = SchemaSerializer(core_schema.with_info_after_validator_function(f, core_schema.int_schema()))
+    with pytest.warns(
+        UserWarning,
+        match=r"Expected `int` - serialized value may not be as expected \[input_value='abc', input_type=str\]",
+    ):
         assert s.to_python('abc') == 'abc'
 
 
@@ -597,8 +640,11 @@ def test_function_wrap_preserves_wrapped_serialization():
     def f(value, handler, _info):
         return handler(value)
 
-    s = SchemaSerializer(core_schema.general_wrap_validator_function(f, core_schema.int_schema()))
-    with pytest.warns(UserWarning, match='Expected `int` but got `str` - serialized value may not be as expected'):
+    s = SchemaSerializer(core_schema.with_info_wrap_validator_function(f, core_schema.int_schema()))
+    with pytest.warns(
+        UserWarning,
+        match=r"Expected `int` - serialized value may not be as expected \[input_value='abc', input_type=str\]",
+    ):
         assert s.to_python('abc') == 'abc'
 
 
